@@ -9,7 +9,8 @@ import exifread
 import json
 import time
 import hashlib
-import pandas as pd
+import pyarrow as pa
+import traceback
 
 import fsspec
 from .logger import CappedCounter
@@ -44,9 +45,11 @@ def download_image_with_retry(row, timeout, retries):
 
 
 def compute_key(key, shard_id, oom_sample_per_shard, oom_shard_count):
-    true_key = (10 ** oom_sample_per_shard) * shard_id + key
+    true_key = (10**oom_sample_per_shard) * shard_id + key
     key_format = oom_sample_per_shard + oom_shard_count
-    str_key = "{true_key:0{key_format}d}".format(key_format=key_format, true_key=true_key)
+    str_key = "{true_key:0{key_format}d}".format(  # pylint: disable=consider-using-f-string
+        key_format=key_format, true_key=true_key
+    )
     return str_key
 
 
@@ -82,28 +85,49 @@ class Downloader:
         self.retries = retries
 
     def __call__(
-        self, row,
+        self,
+        row,
     ):
         try:
             return self.download_shard(row)
         except Exception as err:  # pylint: disable=broad-except
-            print(err)
+            traceback.print_exc()
+            print(f"shard {row[0]} failed with error {err}")
             return (False, 0, 0, 0, 0, 0, None)
 
     def download_shard(
-        self, row,
+        self,
+        row,
     ):
         """Function to start an image downloading in one process"""
 
         shard_id, shard_file = row
+        start_time = time.time()
 
         fs, shard_path = fsspec.core.url_to_fs(shard_file)
         with fs.open(shard_path, "rb") as f:
-            df = pd.read_feather(f)
-        shard_to_dl = list(enumerate(df[self.column_list].to_records(index=False).tolist()))
+            df = pa.ipc.open_file(f).read_all()
+        schema = df.schema
+        schema = (
+            schema.append(pa.field("key", pa.string()))
+            .append(pa.field("status", pa.string()))
+            .append(pa.field("error_message", pa.string()))
+            .append(pa.field("width", pa.int32()))
+            .append(pa.field("height", pa.int32()))
+            .append(pa.field("original_width", pa.int32()))
+            .append(pa.field("original_height", pa.int32()))
+        )
+        if self.extract_exif:
+            schema = schema.append(pa.field("exif", pa.string()))
+
+        if self.compute_md5:
+            schema = schema.append(pa.field("md5", pa.string()))
+
+        pydict = df.select(self.column_list).to_pydict()
+        shard_to_dl = list(enumerate(zip(*(pydict[col] for col in self.column_list))))
+        del pydict
         del df
 
-        start_time = time.perf_counter()
         status_dict = CappedCounter()
 
         count = len(shard_to_dl)
@@ -120,16 +144,24 @@ class Downloader:
 
         def data_generator():
             for e in key_url_list:
-                semaphore.acquire()
+                semaphore.acquire()  # pylint: disable=consider-using-with
                 yield e
 
         loader = data_generator()
 
-        sample_writer = self.sample_writer_class(shard_id, self.output_folder, self.save_caption, self.oom_shard_count)
+        # give schema to writer
+        sample_writer = self.sample_writer_class(
+            shard_id,
+            self.output_folder,
+            self.save_caption,
+            self.oom_shard_count,
+            schema,
+        )
         oom_sample_per_shard = math.ceil(math.log10(self.number_sample_per_shard))
         with ThreadPool(self.thread_count) as thread_pool:
             for key, img_stream, error_message in thread_pool.imap_unordered(
-                lambda x: download_image_with_retry(x, timeout=self.timeout, retries=self.retries), loader
+                lambda x: download_image_with_retry(x, timeout=self.timeout, retries=self.retries),
+                loader,
             ):
                 try:
                     _, sample_data = shard_to_dl[key]
@@ -154,11 +186,22 @@ class Downloader:
                         status_dict.increment(error_message)
                         meta["status"] = status
                         sample_writer.write(
-                            None, str_key, sample_data[caption_indice] if caption_indice is not None else None, meta
+                            None,
+                            str_key,
+                            sample_data[caption_indice] if caption_indice is not None else None,
+                            meta,
                         )
                         semaphore.release()
                         continue
-                    (img, width, height, original_width, original_height, error_message,) = self.resizer(img_stream)
+                    img_stream.seek(0)
+                    (
+                        img,
+                        width,
+                        height,
+                        original_width,
+                        original_height,
+                        error_message,
+                    ) = self.resizer(img_stream)
                     if error_message is not None:
                         failed_to_resize += 1
                         status = "failed_to_resize"
@@ -166,7 +209,10 @@ class Downloader:
                         meta["status"] = status
                         meta["error_message"] = error_message
                         sample_writer.write(
-                            None, str_key, sample_data[caption_indice] if caption_indice is not None else None, meta
+                            None,
+                            str_key,
+                            sample_data[caption_indice] if caption_indice is not None else None,
+                            meta,
                         )
                         img_stream.close()
                         del img_stream
@@ -178,6 +224,7 @@ class Downloader:
 
                     if self.extract_exif:
                         try:
+                            img_stream.seek(0)
                             exif = json.dumps(
                                 {
                                     k: str(v).strip()
@@ -190,6 +237,7 @@ class Downloader:
                         meta["exif"] = exif
 
                     if self.compute_md5:
+                        img_stream.seek(0)
                         meta["md5"] = hashlib.md5(img_stream.read()).hexdigest()
 
                     meta["status"] = status
@@ -201,10 +249,14 @@ class Downloader:
                     del img_stream
 
                     sample_writer.write(
-                        img, str_key, sample_data[caption_indice] if caption_indice is not None else None, meta,
+                        img,
+                        str_key,
+                        sample_data[caption_indice] if caption_indice is not None else None,
+                        meta,
                     )
                 except Exception as err:  # pylint: disable=broad-except
-                    print(err)
+                    traceback.print_exc()
+                    print(f"Sample {key} failed to download: {err}")
                 semaphore.release()
 
             sample_writer.close()
@@ -212,7 +264,7 @@ class Downloader:
             thread_pool.join()
             del thread_pool
 
-        end_time = time.perf_counter()
+        end_time = time.time()
         write_stats(
             self.output_folder,
             shard_id,
