@@ -5,14 +5,15 @@ import time
 from collections import Counter
 import fsspec
 import json
-from multiprocessing import Process, Queue
+import multiprocessing
 import queue
+import traceback
 
 
 class CappedCounter:
     """Maintain a counter with a capping to avoid memory issues"""
 
-    def __init__(self, max_size=10 ** 5):
+    def __init__(self, max_size=10**5):
         self.max_size = max_size
         self.counter = Counter()
 
@@ -36,7 +37,7 @@ class CappedCounter:
         return self.counter
 
     @classmethod
-    def load(cls, d, max_size=10 ** 5):
+    def load(cls, d, max_size=10**5):
         c = CappedCounter(max_size)
         c.counter = Counter(d)
         return c
@@ -45,10 +46,9 @@ class CappedCounter:
 class Logger:
     """logger which logs when number of calls reaches a value or a time interval has passed"""
 
-    def __init__(self, processes_count=1, min_interval=0):
-        """Log only every processes_count and if min_interval (seconds) have elapsed since last log"""
+    def __init__(self, min_interval=0):
+        """Log only every if min_interval (seconds) have elapsed since last log"""
         # wait for all processes to return
-        self.processes_count = processes_count
         self.processes_returned = 0
         # min time (in seconds) before logging a new table (avoids too many logs)
         self.min_interval = min_interval
@@ -60,7 +60,7 @@ class Logger:
 
     def __call__(self, *args, **kwargs):
         self.processes_returned += 1
-        if self.processes_returned % self.processes_count == 0 and time.perf_counter() - self.last > self.min_interval:
+        if time.perf_counter() - self.last > self.min_interval:
             self.do_log(*args, **kwargs)
             self.last = time.perf_counter()
             self.last_call_logged = True
@@ -74,7 +74,7 @@ class Logger:
 
     def sync(self):
         """Ensure last call is logged"""
-        if not self.last_call_logged:
+        if not self.last_call_logged and self.last_args is not None:
             self.do_log(*self.last_args, **self.last_kwargs)
             # reset for next file
             self.processes_returned = 0
@@ -86,7 +86,8 @@ class SpeedLogger(Logger):
     def __init__(self, prefix, enable_wandb, **logger_args):
         super().__init__(**logger_args)
         self.prefix = prefix
-        self.start = time.perf_counter()
+        self.start_time = float("+inf")
+        self.end_time = float("-inf")
         self.count = 0
         self.success = 0
         self.failed_to_download = 0
@@ -94,17 +95,22 @@ class SpeedLogger(Logger):
         self.enable_wandb = enable_wandb
 
     def __call__(
-        self, duration, count, success, failed_to_download, failed_to_resize
+        self, count, success, failed_to_download, failed_to_resize, start_time, end_time
     ):  # pylint: disable=arguments-differ
         self.count += count
         self.success += success
         self.failed_to_download += failed_to_download
         self.failed_to_resize += failed_to_resize
-        super().__call__(duration, self.count, self.success, self.failed_to_download, self.failed_to_resize)
+        self.start_time = min(start_time, self.start_time)
+        self.end_time = max(end_time, self.end_time)
+        super().__call__(
+            self.count, self.success, self.failed_to_download, self.failed_to_resize, self.start_time, self.end_time
+        )
 
     def do_log(
-        self, duration, count, success, failed_to_download, failed_to_resize
+        self, count, success, failed_to_download, failed_to_resize, start_time, end_time
     ):  # pylint: disable=arguments-differ
+        duration = end_time - start_time
         img_per_sec = count / duration
         success_ratio = 1.0 * success / count
         failed_to_download_ratio = 1.0 * failed_to_download / count
@@ -172,10 +178,14 @@ def write_stats(
         "failed_to_download": failed_to_download,
         "failed_to_resize": failed_to_resize,
         "duration": end_time - start_time,
+        "start_time": start_time,
+        "end_time": end_time,
         "status_dict": status_dict.dump(),
     }
     fs, output_path = fsspec.core.url_to_fs(output_folder)
-    shard_name = "{shard_id:0{oom_shard_count}d}".format(shard_id=shard_id, oom_shard_count=oom_shard_count)
+    shard_name = "{shard_id:0{oom_shard_count}d}".format(  # pylint: disable=consider-using-f-string
+        shard_id=shard_id, oom_shard_count=oom_shard_count
+    )
     json_file = f"{output_path}/{shard_name}_stats.json"
     with fs.open(json_file, "w") as f:
         json.dump(stats, f, indent=4)
@@ -183,34 +193,32 @@ def write_stats(
 
 # https://docs.python.org/3/library/multiprocessing.html
 # logger process that reads stats files regularly, aggregates and send to wandb / print to terminal
-class LoggerProcess(Process):
+class LoggerProcess(multiprocessing.context.SpawnProcess):
     """Logger process that reads stats files regularly, aggregates and send to wandb / print to terminal"""
 
-    def __init__(self, output_folder, enable_wandb, wandb_project, config_parameters, processes_count, log_interval=60):
+    def __init__(self, output_folder, enable_wandb, wandb_project, config_parameters, log_interval=5):
         super().__init__()
         self.log_interval = log_interval
         self.enable_wandb = enable_wandb
-        self.fs, self.output_path = fsspec.core.url_to_fs(output_folder)
+        self.output_folder = output_folder
         self.stats_files = set()
         self.wandb_project = wandb_project
+        self.start_shard_id = 0
         self.config_parameters = config_parameters
-        self.processes_count = processes_count
-        self.q = Queue()
+        ctx = multiprocessing.get_context("spawn")
+        self.q = ctx.Queue()
 
     def run(self):
         """Run logger process"""
 
+        fs, output_path = fsspec.core.url_to_fs(self.output_folder, use_listings_cache=False)
+
         if self.enable_wandb:
             self.current_run = wandb.init(project=self.wandb_project, config=self.config_parameters, anonymous="allow")
         else:
             self.current_run = None
-        self.total_speed_logger = SpeedLogger(
-            "total", processes_count=self.processes_count, enable_wandb=self.enable_wandb
-        )
-        self.status_table_logger = StatusTableLogger(
-            processes_count=self.processes_count, enable_wandb=self.enable_wandb
-        )
-        start_time = time.perf_counter()
+        self.total_speed_logger = SpeedLogger("total", enable_wandb=self.enable_wandb)
+        self.status_table_logger = StatusTableLogger(enable_wandb=self.enable_wandb)
         last_check = 0
         total_status_dict = CappedCounter()
         while True:
@@ -225,7 +233,10 @@ class LoggerProcess(Process):
 
             try:
                 # read stats files
-                stats_files = self.fs.glob(self.output_path + "/*.json")
+                stats_files = fs.glob(output_path + "/*.json")
+
+                # filter out files that have an id smaller than start_shard_id
+                stats_files = [f for f in stats_files if int(f.split("/")[-1].split("_")[0]) >= self.start_shard_id]
 
                 # get new stats files
                 new_stats_files = set(stats_files) - self.stats_files
@@ -236,25 +247,30 @@ class LoggerProcess(Process):
 
                 # read new stats files
                 for stats_file in new_stats_files:
-                    with self.fs.open(stats_file, "r") as f:
-                        stats = json.load(f)
-                        SpeedLogger("worker", enable_wandb=self.enable_wandb)(
-                            duration=stats["duration"],
-                            count=stats["count"],
-                            success=stats["successes"],
-                            failed_to_download=stats["failed_to_download"],
-                            failed_to_resize=stats["failed_to_resize"],
-                        )
-                        self.total_speed_logger(
-                            duration=time.perf_counter() - start_time,
-                            count=stats["count"],
-                            success=stats["successes"],
-                            failed_to_download=stats["failed_to_download"],
-                            failed_to_resize=stats["failed_to_resize"],
-                        )
-                        status_dict = CappedCounter.load(stats["status_dict"])
-                        total_status_dict.update(status_dict)
-                        self.status_table_logger(total_status_dict, self.total_speed_logger.count)
+                    with fs.open(stats_file, "r") as f:
+                        try:
+                            stats = json.load(f)
+                            SpeedLogger("worker", enable_wandb=self.enable_wandb)(
+                                count=stats["count"],
+                                success=stats["successes"],
+                                failed_to_download=stats["failed_to_download"],
+                                failed_to_resize=stats["failed_to_resize"],
+                                start_time=stats["start_time"],
+                                end_time=stats["end_time"],
+                            )
+                            self.total_speed_logger(
+                                count=stats["count"],
+                                success=stats["successes"],
+                                failed_to_download=stats["failed_to_download"],
+                                failed_to_resize=stats["failed_to_resize"],
+                                start_time=stats["start_time"],
+                                end_time=stats["end_time"],
+                            )
+                            status_dict = CappedCounter.load(stats["status_dict"])
+                            total_status_dict.update(status_dict)
+                            self.status_table_logger(total_status_dict, self.total_speed_logger.count)
+                        except Exception as err:  # pylint: disable=broad-except
+                            print(f"failed to parse stats file {stats_file}", err)
 
                     self.stats_files.add(stats_file)
                 last_check = time.perf_counter()
@@ -263,7 +279,8 @@ class LoggerProcess(Process):
                     self.finish()
                     return
             except Exception as e:  # pylint: disable=broad-except
-                print(e)
+                traceback.print_exc()
+                print("logger error", e)
                 self.finish()
                 return
 
