@@ -5,7 +5,13 @@ import fire
 import logging
 from .logger import LoggerProcess
 from .resizer import Resizer
-from .writer import WebDatasetSampleWriter, FilesSampleWriter, ParquetSampleWriter
+from .writer import (
+    WebDatasetSampleWriter,
+    FilesSampleWriter,
+    ParquetSampleWriter,
+    TFRecordSampleWriter,
+    DummySampleWriter,
+)
 from .reader import Reader
 from .downloader import Downloader
 from .distributor import multiprocessing_distributor, pyspark_distributor
@@ -44,6 +50,7 @@ def download(
     distributor: str = "multiprocessing",
     subjob_size: int = 1000,
     retries: int = 0,
+    disable_all_reencoding: bool = False,
 ):
     """Download is the main entry point of img2dataset, it uses multiple processes and download multiple files"""
     config_parameters = dict(locals())
@@ -57,8 +64,7 @@ def download(
     output_folder = make_path_absolute(output_folder)
     url_list = make_path_absolute(url_list)
 
-    logger_process = LoggerProcess(output_folder, enable_wandb, wandb_project, config_parameters, processes_count)
-    logger_process.start()
+    logger_process = LoggerProcess(output_folder, enable_wandb, wandb_project, config_parameters)
 
     tmp_path = output_folder + "/_tmp"
     fs, tmp_dir = fsspec.core.url_to_fs(tmp_path)
@@ -83,14 +89,17 @@ def download(
         fs.mkdir(output_path)
         start_shard_id = 0
     else:
-        existing_top_level_files = [x for x in fs.glob(output_path + "/*") if x != tmp_path]
+        existing_top_level_files = [x for x in fs.glob(output_path + "/*") if x != tmp_dir and "stats" not in x]
         if len(existing_top_level_files) == 0:
             start_shard_id = 0
         else:
             start_shard_id = (
-                max([int(x.split("/")[-1].split(".")[0]) for x in existing_top_level_files if x != tmp_path]) + 1
+                max([int(x.split("/")[-1].split(".")[0]) for x in existing_top_level_files if x != tmp_dir]) + 1
             )
 
+    logger_process.start_shard_id = start_shard_id
+    logger_process.start()
+
     reader = Reader(
         url_list,
         input_format,
@@ -108,6 +117,10 @@ def download(
         sample_writer_class = ParquetSampleWriter  # type: ignore
     elif output_format == "files":
         sample_writer_class = FilesSampleWriter  # type: ignore
+    elif output_format == "tfrecord":
+        sample_writer_class = TFRecordSampleWriter  # type: ignore
+    elif output_format == "dummy":
+        sample_writer_class = DummySampleWriter  # type: ignore
 
     resizer = Resizer(
         image_size=image_size,
@@ -117,6 +130,7 @@ def download(
         downscale_interpolation=downscale_interpolation,
         encode_quality=encode_quality,
         skip_reencode=skip_reencode,
+        disable_all_reencoding=disable_all_reencoding,
     )
 
     downloader = Downloader(
@@ -143,7 +157,10 @@ def download(
         raise ValueError(f"Distributor {distributor} not supported")
 
     distributor_fn(
-        processes_count, downloader, reader, subjob_size,
+        processes_count,
+        downloader,
+        reader,
+        subjob_size,
     )
     logger_process.join()
     fs.rm(tmp_dir, recursive=True)
